#!/usr/bin/env python3
# -*- coding: utf-8 -*-

CHANGELOG = {
    "2019.10.7": {
        "semvar": "0.0.1",
        "title": "Initial release that does nothing!",
        "changes": [
            ("add", "packaging stuff")
        ]
    },
    "2019.10.26": {
        "semvar": "0.1.0",
        "title": "First release!",
        "changes": [
            ("add", "download"),
            ("add", "unzip"),
            ("add", "config"),
            ("add", "backup"),
            ("add", "history"),
            ("add", "update"),
            ("add", "login"),
            ("add", "info"),
            ("add", "clean"),
            ("add", "support user.conf")
        ],
        "changes-dev": [
            ("add", "ReadMe.md")
            ("add", "travis-ci"),
            ("add", "logging"),
            ("add", "dev scripts to build and publish"),
            ("add", "spellcheck")
        ]
    }
}
